#!/usr/local/bin/php -q
<?php
class dm_twitter
{
	private $tweeter;
	private $statuses;
	private $json_url;
	private $filename;

	function __construct()
	{
		// initialize the variables - username, number of statuses to retrieve and the url to the twitter json feed
		$this->tweeter	= 'DanielMenjivar';
		$this->statuses	= 6;
		$this->json_url	= 'http://api.twitter.com/1/statuses/user_timeline/'.$this->tweeter.'.json?count='.$this->statuses;
		$this->filename	= 'public_html/twitter.json';
	}

	private function get_json ()
	{
		// use cURL to get the json feed
		$ch = curl_init($this->json_url);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($ch, CURLOPT_VERBOSE, 1);
		curl_setopt($ch, CURLOPT_NOBODY, 0);
		curl_setopt($ch, CURLOPT_HEADER, false);
		curl_setopt($ch, CURLOPT_FOLLOWLOCATION,1);

		// try getting the data from twitter and return data if successful, otherwise, just exit the script
		try
		{
			$data		= curl_exec($ch);
			$response	= curl_getinfo($ch);
			curl_close($ch);

			if ( intval( $response['http_code'] ) == 200 )
			{
				return $data;
			} else {
				exit();
			}
		} catch (Exception $e) {
			exit();
		}
	}

	private function reformat_json()
	{
		// get the json feed and decode it
		$data = json_decode($this->get_json());

		// create a formatted array with all the tweets
		$index = 0;
		foreach ($data as $tweet)
		{
			$tweets[$index]->date	= $this->TimeAgo($tweet->created_at);
			$tweets[$index]->link	= 'http://twitter.com/'.$this->tweeter.'/statuses/'.$tweet->id;
			$tweets[$index]->text	= $this->linkify_text($tweet->text);
			if ($tweet->place->full_name)
			{
				$tweets[$index]->place	= $tweet->place->full_name;
				$tweets[$index]->maplink= 'http://maps.google.com/maps?&amp;q='.implode(",",array_reverse($tweet->coordinates->coordinates));
			}
			$index ++;
		}
		// we need to send some user info too!
		$twitter_data->userlink	=	'http://twitter.com/'.$this->tweeter;
		$twitter_data->tweets	=	$tweets;
		return json_encode($twitter_data);
	}

	public function save_json_file()
	{
		$content = $this->reformat_json();
		$filename = $this->filename;

		// Let's make sure the file exists and is writable first.
		if (is_writable($filename))
		{
			// open the file for writing
			if (!$handle = fopen($filename, 'w'))
			{
				echo "Cannot open file ($filename)";
				exit();
			}

			// Write $content to our opened file.
			if (fwrite($handle, $content) === FALSE)
			{
				echo "Cannot write to file ($filename)";
				exit();
			}

			// close the file
			fclose($handle);

			return true;
		}
	}

	private function linkify_text($raw_text)
	{
		// first set output to the value we received when calling this function
		$output = $raw_text;

		// create xhtml safe text (mostly to be safe of ampersands)
		$output = htmlentities($raw_text, ENT_NOQUOTES, 'UTF-8');

		// parse urls
		$pattern = '/([A-Za-z]+:\/\/[A-Za-z0-9-_]+\.[A-Za-z0-9-_:%&\?\/.=]+)/i';
		$replacement = '<a href="${1}" rel="external">${1}</a>';
		$output = preg_replace($pattern, $replacement, $output);

		// parse usernames
		$pattern = '/[@]+([A-Za-z0-9-_]+)/';
		$replacement = '<a href="http://twitter.com/${1}" rel="external">@${1}</a>';
		$output = preg_replace($pattern, $replacement, $output);

		// parse hashtags
		$pattern = '/[#]+([A-Za-z0-9-_]+)/';
		$replacement = '<a href="http://search.twitter.com/search?q=%23${1}" rel="external">#${1}</a>';
		$output = preg_replace($pattern, $replacement, $output);

		return $output;
	}

	//$datefrom is the timestamp for the content, and you can leave the $dateto value to see the current delay
	private function TimeAgo($datefrom,$dateto=-1)
	{
		// convert the $datefrom into the format we need first
			$datefrom	= strtotime($datefrom);

		// Defaults and assume if 0 is passed in that its an error rather than the epoch
			if($datefrom<=0) { return "a long time ago"; }
			if($dateto==-1) { $dateto = time()+1; }

		// Calculate the difference in seconds betweeen the two timestamps
			$difference = $dateto - $datefrom;

		switch ($difference)
		{
			case ($difference < 60): // less than 60 seconds
				return "less than a minute ago";
				break;
			case ($difference >= 60 && $difference<60*60): // between 60 seconds and 60 minutes
				$datediff = floor($difference / 60);
				$res = ($datediff==1) ? "about $datediff minute ago" : "about $datediff minutes ago";
				return $res;
				break;
			case ($difference >= 60*60 && $difference<60*60*24): // between 1 hour and 24 hours
				$datediff = floor($difference / 60 / 60);
				$res = ($datediff==1) ? "about $datediff hour ago" : "about $datediff hours ago";
				return $res;
				break;
			case ($difference >= 60*60*24): // greater than 1 day
				$datediff = floor($difference / 60 / 60 / 24);
				$res = ($datediff==1) ? "about $datediff day ago" : "about $datediff days ago";
				return $res;
				break;
		}
	}
}

// this function looks for all previous processes running this script, and if they exist, it kills the process :-)
function kill_previous()
{
	// set the path to this file to the $file variable
	$file = __FILE__;

	// run the command line to get a list of all the php5 processes and assign it to $processes
	exec("ps -C php5 -o pid= -o %a",$processes);

	// foreach PHP5 process, if it contains the name of this file, then let's kill that process
	foreach ($processes as $key=>$process)
	{
		$pid	=	intval($process);
		if (strstr($process,$file) && $pid != getmypid()){
			exec("kill $pid");
		}
	}
}
// end of kill_previous function

	// let's kill all previous processes running this script, and run it again
	kill_previous();
	$twitter = new dm_twitter;
	$twitter->save_json_file();
?>